// Chat() - Kernel Based - for TI-89
// by DLL - laurent@etive.net

/* For Kernel-Less Program
#define OPTIMIZE_ROM_CALLS				// Use Speed Optimization
#define SAVE_SCREEN								// Save/Restore LCD Contents
*/

#define USE_KERNEL            		// Make Kernel Program
#include <link.h>									// Link Port
#include <graph.h>
#include <kbd.h>									// Keyboard
#include <system.h>
#include <statline.h>							// Status Line

int _ti89;                    		// Produce .89Z File

char _comment[] = "DLL Chat 1.0";

#define max_x 160
#define max_y 45

// unsigned: positive only
typedef struct {
	unsigned int x;
	unsigned int y;
} chr_pos;

// Defines functions
chr_pos ShowChar(chr_pos pos, unsigned int min_y, char *chr);
chr_pos DoBackspace(chr_pos pos, unsigned int min_y);
chr_pos GoToNextLne(chr_pos pos, unsigned int min_y);
chr_pos SendChar(chr_pos pos, char *chr);

void _main(void) {
	chr_pos snd_pos, rcv_pos;
	unsigned int key = 0;
	unsigned int exit = 0;
	unsigned char buffer;
	
	// SPLASH SCREEN
	ClrScr();											// Clear the screen
	
	FontSetSys(F_8x10);
	DrawStr(48, 30, "DLL Chat", A_NORMAL);
	FontSetSys(F_6x8);
	DrawStr(93, 41, "1.0", A_NORMAL);
	DrawClipRect(MakeWinRect(38, 23, 121, 52), ScrRect, A_NORMAL);
	FontSetSys(F_4x6);
	DrawStr(48, 43, "for TI-89", A_NORMAL);
	DrawStr(32, 70, "by Laurent Debacker as DLL", A_NORMAL); // 26
	DrawStr(21, 77, "http:/www15.brinkster.com/DLL/", A_NORMAL);	// 30
	DrawStr(47, 84, "laurent@etive.com", A_NORMAL);	// 17
	
	ngetchx();										// Wait any keys press
	
	// CHAT FORM
	ClrScr();
	
	snd_pos.x = 0; snd_pos.y = 0;
	rcv_pos.x = 0; rcv_pos.y = 48;
	
	DrawLine(0, 46, 159, 46, A_NORMAL);
	DrawLine(0, 93, 159, 93, A_NORMAL);
	
	DrawStr(0, 95, "DLL Chat", A_NORMAL);
	
	FontSetSys(F_6x8);
	
	OSLinkOpen();
	
	DrawChar(snd_pos.x, snd_pos.y, '_', A_REPLACE);
	DrawChar(rcv_pos.x, rcv_pos.y, '_', A_REPLACE);
	
	while (exit == 0) {
	
		while (OSReadLinkBlock(&buffer, 1) != 0) {
			//printf("%d",buffer);
			//break;
  		switch (buffer) {
				case 16:		// Enter
		  		rcv_pos = GoToNextLne(rcv_pos, 48);
					break;
 				case 17:		// Backspace
					rcv_pos = DoBackspace(rcv_pos, 48);
					break;
				case 18:		// Other side has quit
					exit = 1;
					break;
				default:
		  		rcv_pos = ShowChar(rcv_pos, 48, &buffer);
			}
			DrawChar(rcv_pos.x, rcv_pos.y, '_', A_REPLACE);
  	}
  	
  	//Check keyboard		
		key = kbhit();
		if (key != 0) {
			GKeyFlush();
			ST_busy(ST_NORMAL);
			switch (key) {
		  	case KEY_ESC:
	  			exit = 1;
	  			break;
		  	case KEY_F1:
			  	snd_pos = SendChar(snd_pos, "?");	// 63 -> 3Fh
			  	break;
	  		case KEY_F2:
			  	snd_pos = SendChar(snd_pos, "!");	// 33 -> 21h
			  	break;
	  		case KEY_F3:
		  		snd_pos = SendChar(snd_pos, "=");	// 58 -> 3Ah
		  		break;
		  	case KEY_F4:
			  	snd_pos = SendChar(snd_pos, ";");	// 59 -> 3Bh
			  	break;
	  		case KEY_F5:
			  	snd_pos = SendChar(snd_pos, "#");	// 35 -> 23h
			  	break;
	  		case KEY_BACKSPACE:									// <-
	  			snd_pos = DoBackspace(snd_pos, 0);
		  		OSWriteLinkBlock("\x11", 1);			// 17 -> 11h
			  	break;
			  case KEY_ENTER:
			  	snd_pos = GoToNextLne(snd_pos, 0);
			  	OSWriteLinkBlock("\x10", 1);			// 16 -> 10h
			  	break;
				default:
					buffer = key;
					snd_pos = SendChar(snd_pos, &buffer);
			}
			DrawChar(snd_pos.x, snd_pos.y, '_', A_REPLACE);
		}
		
	}
	
	OSWriteLinkBlock("\x12", 1);	// 18 -> 12h
	OSLinkClose();
}

chr_pos ShowChar(chr_pos pos, unsigned int min_y, char *chr) {
	chr_pos npos;
	
	DrawChar(pos.x, pos.y, *chr, A_REPLACE);
	
	npos.x = pos.x + 6;
 	if (npos.x > max_x - 6) {
 		npos.x = 0;
 		npos.y = pos.y + 8;
 		if (npos.y > max_y + min_y - 8) npos.y = min_y;
 	} else {
 		npos.y = pos.y;
	}
	
	return npos;
}

chr_pos DoBackspace(chr_pos pos, unsigned int min_y) {
	chr_pos npos;
	
	DrawChar(pos.x, pos.y, ' ', A_REPLACE);
	
	if (pos.x == 0) {
 		npos.x = 150;
 		if (pos.y == min_y) {
	 		npos.y = min_y + 32;
		} else {
			npos.y = pos.y - 8;
 		}
 	} else {
		npos.x = pos.x - 6;
		npos.y = pos.y;
	}
	
	return npos;
}

chr_pos GoToNextLne(chr_pos pos, unsigned int min_y) {
	chr_pos npos;
	unsigned int x;
	
	npos.x = 0;
	npos.y = pos.y + 8;
	if (npos.y > max_y + min_y - 8) npos.y = min_y;
	
	for (x = pos.x; x < max_x; x = x + 6) {
		DrawChar(x, pos.y, ' ', A_REPLACE);
	}
	
	return npos;
}

chr_pos SendChar(chr_pos pos, char *chr) {
	chr_pos npos;
	
	OSWriteLinkBlock(chr, 1);
	npos = ShowChar(pos, 0, chr);
	
	return npos;
}
